//package net.comp_lot.htmlBuilder;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.List;

/**
 * プログラムのコードからSVGの画像を作る
 * インデントはTabで行われている必要がある
 * 文の後ろにTabがあると失敗する
 * @author pochi
 */
public class SvgBuilder {
	/**
	 * 保存先フォルダ
	 */
	private static final String OUTPUT = "output/";
	
	/**
	 * 画像を作って保存する
	 * @param path コードのパス
	 */
	public static void buildSvg(String path) {
		File file = new File(path);
		List<String> codes = new ArrayList<String>();
		try (BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(file),"UTF-8"))) {
			String line = br.readLine();
			while(line != null){
				codes.add(line);
				line = br.readLine();
			}
			br.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		try (BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(new File(OUTPUT + file.getName() + ".svg")),"UTF-8"))) {
			bw.write(buildSvg(codes.remove(0), codes.toArray(new String[codes.size()])));
			bw.flush();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 画像を作る
	 * @param codes 1行ずつになったコード
	 * @return 画像の文字列
	 */
	private static String buildSvg(String name, String...codes) {
		int size = String.valueOf(codes.length).length();
		
		StringBuilder sb = new StringBuilder();
		sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<svg width=\"900\" height=\"");
		sb.append((codes.length+2) * 25);
		sb.append("\"  viewBox=\"0 0 900 ");
		sb.append((codes.length+2) * 25);
		sb.append("\" xmlns=\"http://www.w3.org/2000/svg\">\n");
		
		sb.append("\t<text x=\"20\" y=\"25\" font-size=\"22\" stroke=\"green\" text-anchor=\"start\"  stroke-width=\"0.5\" >");
		sb.append(name);
		sb.append("</text>\n");
		
		int i = 0;
		boolean inComment = false;
		for (String line : codes) {
			line = line.replaceAll("&", "&amp;");
			line = line.replaceAll("<", "&lt;");
			line = line.replaceAll(">", "&gt;");
			
			// 行番号
			addTextStart(sb, 0, ++i * 25 + 35, "teal");
			sb.append(i);
			sb.append("</text>\n");
			
			// 本文
			addTextStart(sb, (line.lastIndexOf("\t") + 1) * 40 + size * 20, i * 25 + 35, "black");
			sb.append("\n\t\t");
			String trim = line.trim();
			
			// 色分けの処理
			char before = ' ';
			boolean comment = false;
			boolean string = false;
			if (inComment)
				sb.append("<tspan stroke=\"green\">");
			for (int j=0;j<trim.length();j++) {
				char c = trim.charAt(j);
				switch (c) {
				case '"':
					if (comment || inComment) {
						sb.append(c);
						break;
					}
					char next = ' ';
					if (j+1 < trim.length())
						next = trim.charAt(j+1);
					if (before != '\\' && !(before == '\'' && next == '\'')) {
						if (!string) {
							sb.append("<tspan stroke=\"blue\">");
							sb.append(c);
							string = true;
						} else {
							sb.append(c);
							sb.append("</tspan>");
							string = false;
						}
					} else {
						sb.append(c);
					}
					break;
				case '/':
					if (string) {
						sb.append(c);
						break;
					}
					if (before == '/' && !inComment) {
						sb.insert(sb.length()-1, "<tspan stroke=\"green\">");
						sb.append(c);
						comment = true;
					} else if (before == '*' && inComment && !comment) {
						sb.append(c);
						sb.append("</tspan>");
						inComment = false;
					} else {
						sb.append(c);
					}
					break;
				case '*':
					if (string) {
						sb.append(c);
						break;
					}
					if (before == '/' && !comment) {
						sb.insert(sb.length()-1, "<tspan stroke=\"green\">");
						sb.append(c);
						inComment = true;
					} else {
						sb.append(c);
					}
					break;
				default:
					sb.append(c);
				}
				before = c;
			}
			if (comment || inComment)
				sb.append("</tspan>");
			
			sb.append("\n\t</text>\n");
		}
		sb.append("</svg>");
		return sb.toString();
	}
	
	/**
	 * テキストの開始タグを追加する
	 * @param sb 対象
	 * @param x 位置
	 * @param y 位置
	 * @param color 色
	 */
	private static void addTextStart(StringBuilder sb, int x, int y, String color) {
		sb.append("\t<text x=\"");
		sb.append(x);
		sb.append("\" y=\"");
		sb.append(y);
		sb.append("\" font-size=\"20\" stroke=\"");
		sb.append(color);
		sb.append("\" text-anchor=\"start\"  stroke-width=\"0.5\" >");
	}
}
